// ---------------------------------------------------------------------
// exemple-commutation-processus.c
// Fichier d'exemple du livre "Solutions Temps-Reel sous Linux"
// (C) 2012-2015 - Christophe BLAESS
// http://christophe.blaess.fr
// ---------------------------------------------------------------------

#include <fcntl.h>
#include <sched.h>
#include <semaphore.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/time.h>
#include <sys/wait.h>

static sem_t * semaphore = NULL;

#define Nb_SWITCHES_MAX  1000
static int * nb_switches = 0;
static long int * durations = NULL;

static struct timeval * before_switch = NULL;

void high_priority_child (void)
{
	struct timeval after_switch;
	usleep(100000); // 100 ms
	while ((*nb_switches) < Nb_SWITCHES_MAX) {
		sem_wait(semaphore);
		gettimeofday(& after_switch, NULL);
		durations[(*nb_switches)]  = after_switch.tv_sec - before_switch->tv_sec;
		durations[(*nb_switches)] *= 1000000;
		durations[(*nb_switches)] += after_switch.tv_usec - before_switch->tv_usec;
		(*nb_switches) ++;
	}
}


void low_priority_child (void)
{
	sem_wait(semaphore);
	usleep(200000); // 200 ms
	while ((*nb_switches) < Nb_SWITCHES_MAX) {
		gettimeofday(before_switch, NULL);
		sem_post(semaphore);
		usleep(1000); // 1 ms
	}
	sem_post(semaphore);
}

int main(void)
{	
	int i;
	int md;
	struct sched_param param;

	semaphore = sem_open("/test-01", O_CREAT | O_RDWR, 0600, 1);
	if (semaphore == SEM_FAILED) {
		perror("/test-01");
		exit(EXIT_FAILURE);
	}
	sem_unlink("/test-01");

	md = shm_open("/test-02", O_CREAT | O_RDWR, 0600);
	if (md < 0) {
		perror("/test-02");
		exit(EXIT_FAILURE);
	}
	ftruncate(md, sizeof(struct timeval));
	before_switch = mmap(NULL, sizeof(struct timeval), PROT_READ | PROT_WRITE, MAP_SHARED, md, 0);
	if (before_switch == MAP_FAILED) {
		perror("/test-02");
		exit(EXIT_FAILURE);
	}
	shm_unlink("/test-02");

	md = shm_open("/test-03", O_CREAT | O_RDWR, 0600);
	if (md < 0) {
		perror("/test-03");
		exit(EXIT_FAILURE);
	}
	ftruncate(md, Nb_SWITCHES_MAX * sizeof(long int));
	durations = mmap(NULL, Nb_SWITCHES_MAX * sizeof(long int), PROT_READ | PROT_WRITE, MAP_SHARED, md, 0);
	if (durations == MAP_FAILED) {
		perror("/test-03");
		exit(EXIT_FAILURE);
	}
	shm_unlink("/test-03");
	
	md = shm_open("/test-04", O_CREAT | O_RDWR, 0600);
	if (md < 0) {
		perror("/test-04");
		exit(EXIT_FAILURE);
	}
	ftruncate(md, sizeof(int));
	nb_switches = mmap(NULL, sizeof(int), PROT_READ | PROT_WRITE, MAP_SHARED, md, 0);
	if (nb_switches == MAP_FAILED) {
		perror("/test-04");
		exit(EXIT_FAILURE);
	}
	(* nb_switches) = 0;
	shm_unlink("/test-04");
	mlockall(MCL_CURRENT | MCL_FUTURE);
	if (fork() == 0) {
		/* First child */
		param.sched_priority = 20;
		if (sched_setscheduler(0, SCHED_FIFO, & param) != 0) {
			perror("setscheduler");
			exit(EXIT_FAILURE);
		}
		low_priority_child();
		exit(EXIT_SUCCESS);
	}

	if (fork() == 0) {
		/* Second child */
		param.sched_priority = 80;
		if (sched_setscheduler(0, SCHED_FIFO, & param) != 0) {
			perror("setscheduler");
			exit(EXIT_FAILURE);
		}
		high_priority_child();
		exit(EXIT_SUCCESS);
	}
	waitpid(-1 , NULL, 0);
	waitpid(-1 , NULL, 0);

	for (i = 0; i < Nb_SWITCHES_MAX; i ++)
		printf("%ld\n", durations[i]);

	return EXIT_SUCCESS;
}

