// ---------------------------------------------------------------------
// exemple-commutation-threads.c
// Fichier d'exemple du livre "Solutions Temps-Reel sous Linux"
// (C) 2012-2015 - Christophe BLAESS
// http://christophe.blaess.fr
// ---------------------------------------------------------------------

#include <pthread.h>
#include <sched.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/time.h>

static pthread_mutex_t  mutex  = PTHREAD_MUTEX_INITIALIZER;

#define NB_SWITCHES_MAX  1000
static int nb_switches = 0;
static long int durations[NB_SWITCHES_MAX];

static struct timeval before_switch;


void * high_priority_thread (void * unused)
{
	struct timeval after_switch;

	usleep(100000); // 100 ms
	while (nb_switches < NB_SWITCHES_MAX) {
		pthread_mutex_lock(& mutex);
		gettimeofday(& after_switch, NULL);
		durations[nb_switches]  = after_switch.tv_sec - before_switch.tv_sec;
		durations[nb_switches] *= 1000000;
		durations[nb_switches] += after_switch.tv_usec - before_switch.tv_usec;
		nb_switches ++;
		pthread_mutex_unlock(& mutex);
		usleep(5000); // 5 ms
	}
	return NULL;
}


void * low_priority_thread (void * unused)
{
	pthread_mutex_lock(& mutex);
	usleep(200000); // 200 ms
	while (nb_switches < NB_SWITCHES_MAX) {
		gettimeofday(& before_switch, NULL);
		pthread_mutex_unlock(& mutex);
		pthread_mutex_lock(& mutex);
		usleep(10000); // 10 ms
	}
	pthread_mutex_unlock(& mutex);
	return NULL;
}

int main(void)
{	
	int i;
	int err;
	pthread_attr_t attr;
	struct sched_param param;
	pthread_t thr[2];

	pthread_attr_init (& attr);
	if ((err = pthread_attr_setschedpolicy(& attr, SCHED_FIFO)) != 0) {
		fprintf(stderr, "setschedpolicy: %s\n", strerror(err));
		exit(EXIT_FAILURE);
	}
	if ((err = pthread_attr_setinheritsched(& attr, PTHREAD_EXPLICIT_SCHED)) != 0) {
		fprintf(stderr, "setinheritsched: %s\n", strerror(err));
		exit(EXIT_FAILURE);
	}

	param.sched_priority = 20;
	if ((err = pthread_attr_setschedparam(& attr, & param)) != 0) {
		fprintf(stderr, "setschedparam: %s\n", strerror(err));
		exit(EXIT_FAILURE);
	}
	if ((err = pthread_create(& (thr[0]), & attr, low_priority_thread, NULL)) != 0) {
		fprintf(stderr, "pthread_create: %s\n", strerror(err));
		exit(EXIT_FAILURE);
	}

	mlockall(MCL_CURRENT | MCL_FUTURE);
	param.sched_priority = 80;
	if ((err = pthread_attr_setschedparam(& attr, & param)) != 0) {
		fprintf(stderr, "setschedparam: %s\n", strerror(err));
		exit(EXIT_FAILURE);
	}
	if ((err = pthread_create(& (thr[1]), & attr, high_priority_thread, NULL)) != 0) {
		fprintf(stderr, "pthread_create: %s\n", strerror(err));
		exit(EXIT_FAILURE);
	}

	pthread_join(thr[0], NULL);
	pthread_join(thr[1], NULL);

	for (i = 0; i < NB_SWITCHES_MAX; i ++)
		printf("%ld\n", durations[i]);

	return EXIT_SUCCESS;
}

