// ---------------------------------------------------------------------
// uart-echo-timing.c
// Fichier d'exemple du livre "Solutions Temps-Reel sous Linux"
// (C) 2012-2015 - Christophe BLAESS
// http://christophe.blaess.fr
// ---------------------------------------------------------------------


#include <intrinsics.h>
#include <msp430g2553.h>

void send_char(char c);
void send_long_int(long l);
void delay_ms(unsigned int ms);

int main(void)
{
	long int counter;

	// Stop the watchdog.
	WDTCTL  = WDTPW + WDTHOLD;

	// Leds P1.0 et P1.6 to output, turned off.
	P1DIR |=  BIT0 | BIT6;
	P1OUT &= ~(BIT0 | BIT6);

	// P1.3 button on input with pull-up resistor.
	P1DIR &= ~BIT3;
	P1REN |=  BIT3;
	P1OUT |=  BIT3;

	// Select a 16MHz Basic clock.
	DCOCTL  = 0;
	BCSCTL1 = CALBC1_16MHZ;
	DCOCTL  = CALDCO_16MHZ;

	// Select secondary function for P1.1 (Rx)  and P1.2 (Tx)
	P1SEL   = BIT1 | BIT2;
	P1SEL2  = BIT1 | BIT2;

	// UART, 8bits, no parity, 1 stop, 
	UCA0CTL0 = 0;

	// See MSP430x2xx User Guide (slau144j) p.424:
	// BRCLK = 16MHz
	// Baud rate = 115200
	//  -> UCBRx  = 137
	//  -> UCBRSx = 7
	//  -> UCBRFx = 0 
	UCA0CTL1 |= UCSSEL_2;
	UCA0BR0  = 138;
	UCA0BR1  = 0;
	UCA0MCTL = UCBRS2 + UCBRS1 + UCBRS0;
	UCA0CTL1 &= ~UCSWRST; // USCI Reset.

	IE2 |= UCA0TXIE | UCA0RXIE;

	// Green led on.
	P1OUT |= BIT6;
	// Wait for button press.
	while ((P1IN & BIT3) != 0)
		;
	// Green led off.
	P1OUT &= ~BIT6;

	while (1) {
		counter = 0;
		// Red led on.
		P1OUT |= BIT0;
		// Send a request.
		send_char('A');
		// Wait for an answer.
		while(!(IFG2 & UCA0RXIFG)) {
			counter ++;
		}
		// Red led off.
		P1OUT &= ~BIT0;
		if (UCA0RXBUF != 'A')
			continue;
		// Send the timing.
		send_long_int(counter);
		delay_ms(1);
		send_char('\r');
		delay_ms(1);
		send_char('\n');
		delay_ms(100);
	}

	return 0;
}

void delay_ms(unsigned int ms)
{
	while ((ms--) > 0)
		__delay_cycles(16000);
}

void send_char(char c)
{
	// Wait Tx empty.
	while(!(IFG2 & UCA0TXIFG))
		;
	UCA0TXBUF = c;
}

void send_long_int(long l)
{
	if (l > 10) {
		send_long_int(l / 10);
		delay_ms(1);
	}
	send_char((l%10) + '0');
}


