// ---------------------------------------------------------------------
// exemple-boucle-temps-reel.c
// Fichier d'exemple du livre "Solutions Temps-Reel sous Linux"
// (C) 2012-2015 Christophe BLAESS
// http://christophe.blaess.fr
// ---------------------------------------------------------------------

#define _GNU_SOURCE  // Pour avoir sched_setaffinity
#include <pthread.h>
#include <sched.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

	static long int count = 500000000;
	static pthread_barrier_t barrier;

void * thread_function(void * arg)
{
	long n;
	long cpu = (long) arg;
	cpu_set_t  cpuset;

	// Passer sur le CPU demande.
	CPU_ZERO(& cpuset);
	CPU_SET(cpu, &cpuset);
	if (pthread_setaffinity_np(pthread_self(),
	                           sizeof(cpuset), & cpuset) !=0) {
		perror("pthread_setaffinity_np");
		exit(EXIT_FAILURE);
	}

	pthread_barrier_wait(& barrier);

	// Executer la boucle de comptage.
	for (n = 0; n < count; n ++)
		;
	return NULL;
}


int main(int argc, char * argv[])
{
	long int nb_cpus = sysconf(_SC_NPROCESSORS_ONLN);
	pthread_t * threads;
	long int cpu;
	struct sched_param param;

	// Initialiser la valeur max. du comptage.
	if (argc > 1) {
		if (sscanf(argv[1], "%ld", & count) != 1) {
			fprintf(stderr, "usage: %s [count-value]\n",
			        argv[0]);
			exit(EXIT_FAILURE);
		}
	}

	// Passer en temps-reel.
	param.sched_priority = 99;
	if (sched_setscheduler(0, SCHED_FIFO, & param) != 0) {
		perror("sched_setscheduler");
		exit(EXIT_FAILURE);
	}

	// Initialiser la table des threads.
	threads = calloc(nb_cpus, sizeof(pthread_t));
	if (threads == NULL) {
		perror(argv[0]);
		exit(EXIT_FAILURE);
	}

	// Demarrer les threads.
	pthread_barrier_init(& barrier, NULL, nb_cpus);
	for (cpu = 0; cpu < nb_cpus; cpu ++) {
		if (pthread_create(& (threads[cpu]), NULL,
		                   thread_function, (void *) cpu) != 0) {
			perror(argv[0]);
			exit(EXIT_FAILURE);
		}
	}

	// Attendre la fin des threads.
	for (cpu = 0; cpu < nb_cpus; cpu ++) {
		pthread_join(threads[cpu], NULL);
	}

	free(threads);
	return EXIT_SUCCESS;
}


