// ---------------------------------------------------------------------
// exemple-governor-01.c
// Fichier d'exemple du livre "Solutions Temps-Reel sous Linux"
// (C) 2012-2015 - Christophe BLAESS
// http://christophe.blaess.fr
// ---------------------------------------------------------------------

#define _GNU_SOURCE 
#include <fcntl.h>
#include <sched.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>

#define NB_BUSY_LOOPS 200000
#define NB_CYCLES     10000

static struct timeval tv[NB_CYCLES + 1];

int main(int argc, char * argv[])
{
	int fd;
	int i, j;
	char filename[256];
	long long int duration;
	struct sched_param param;

	if (argc != 2) {
		fprintf(stderr, "usage: %s <governor>\n", argv[0]);
		exit(1);
	}

	sprintf(filename, "/sys/devices/system/cpu/cpu%d/cpufreq/scaling_governor", sched_getcpu());
	fd = open(filename, O_RDWR);
	if (fd < 0) {
		perror(filename);
		exit(2);
	}
	if (write(fd, argv[1], strlen(argv[1])) < 0) {
		perror(argv[1]);
		exit(2);
	}
	close (fd);

	param.sched_priority = 10;
	if (sched_setscheduler(0, SCHED_FIFO, & param) != 0) {
		perror("pthread_attr_setschedpolicy");
		exit(2);
	}

	sleep(1);

	for (i = 0; i < NB_CYCLES; i ++) {
		gettimeofday(& (tv[i]), NULL);
		for (j = 0; j < NB_BUSY_LOOPS; j ++)
			;
	//	usleep(1);
	}
	gettimeofday(& (tv[i]), NULL);

	for (i = 0; i < NB_CYCLES; i ++) {
		duration  = tv[i+1].tv_sec - tv[i].tv_sec;
		duration *= 1000000;
		duration += tv[i+1].tv_usec - tv[i].tv_usec;
		printf("%6lld\n", duration);
	}
	return EXIT_SUCCESS;
}



