// ---------------------------------------------------------------------
// exemple-mutex-01.c
// Fichier d'exemple du livre "Solutions Temps-Reel sous Linux"
// (C) 2012 - Christophe BLAESS
// http://christophe.blaess.fr
// ---------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/mman.h>

#include <alchemy/task.h>
#include <alchemy/mutex.h>
#include <alchemy/timer.h>

static RT_MUTEX  mutex;

#define NB_SWITCHES_MAX  1000
static int nb_switches = 0;
static unsigned long long int before_switch;
static unsigned long long int after_switch;
static unsigned long long int switch_duration[NB_SWITCHES_MAX];


void thread_2 (void * unused)
{
	rt_task_sleep(5000000UL); // 5 ms

	while (nb_switches < NB_SWITCHES_MAX) {
		rt_mutex_acquire(& mutex, TM_INFINITE);
		after_switch = rt_timer_read();
		switch_duration[nb_switches] = after_switch - before_switch;
		nb_switches ++;
		rt_mutex_release(& mutex);
		rt_task_yield();
	}
}


void thread_1 (void * unused)
{
	rt_mutex_acquire(& mutex, TM_INFINITE);

	while (nb_switches < NB_SWITCHES_MAX) {
		rt_task_sleep(10000000UL); // 10 ms
		before_switch = rt_timer_read();
		rt_mutex_release(& mutex);
		rt_task_yield();
		rt_mutex_acquire(& mutex, TM_INFINITE);
	}
	rt_mutex_release(& mutex);
}

int main(void)
{
	int i;
	int err;
	RT_TASK task[2];

	mlockall(MCL_CURRENT|MCL_FUTURE);

	if ((err = rt_mutex_create(& mutex, "Mutex-01")) != 0) {
		fprintf(stderr, "rt_mutex_create:%s\n",
		         strerror(-err));
		exit(EXIT_FAILURE);
	}

	if ((err = rt_task_spawn(&task[0], NULL, 0, 99,
	                  T_JOINABLE, thread_1, NULL) != 0)) {
		fprintf(stderr, "rt_task_spawn:%s\n",
		        strerror(-err));
		exit(EXIT_FAILURE);
	}

	if (rt_task_spawn(&task[1], NULL, 0, 99,
	                  T_JOINABLE, thread_2, NULL) != 0) {
		fprintf(stderr, "rt_task_spawn:%s\n",
		        strerror(-err));
		exit(EXIT_FAILURE);
	}
	rt_task_join(& task[0]);
	rt_task_join(& task[1]);
	rt_mutex_delete(& mutex);

	for (i = 0; i < NB_SWITCHES_MAX; i ++)
		printf("%lld\n", switch_duration[i]/1000);

	return EXIT_SUCCESS;
}

