// ---------------------------------------------------------------------
// exemple-mutex-02.c
// Fichier d'exemple du livre "Solutions Temps-Reel sous Linux"
// (C) 2012-2015 - Christophe BLAESS
// http://christophe.blaess.fr
// ---------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/mman.h>

#include <alchemy/task.h>
#include <alchemy/mutex.h>
#include <alchemy/timer.h>

static RT_MUTEX  mutex;


void thread_1(void *unused);
void thread_2(void * unused);
void thread_3(void * unused);


void thread_1(void *unused)
{
	int err;
	RT_TASK task_2, task_3;

	rt_fprintf(stderr, "T1 starts\n");

	rt_fprintf(stderr, "T1 wait for the mutex\n");
	rt_mutex_acquire(& mutex, TM_INFINITE);
	rt_fprintf(stderr, "T1 holds the mutex\n");

	rt_fprintf(stderr, "T1 starts T3\n");
	if ((err = rt_task_spawn(& task_3, NULL, 0, 30,
	                  T_JOINABLE, thread_3, NULL) != 0)) {
		fprintf(stderr, "rt_task_spawn:%s\n",
		        strerror(-err));
		exit(EXIT_FAILURE);
	}

	rt_fprintf(stderr, "T1 starts T2\n");
	if ((err = rt_task_spawn(& task_2, NULL, 0, 20,
	                  T_JOINABLE, thread_2, NULL) != 0)) {
		fprintf(stderr, "rt_task_spawn:%s\n",
		        strerror(-err));
		exit(EXIT_FAILURE);
	}

	rt_fprintf(stderr, "T1 releases the mutex\n");
	rt_mutex_release(& mutex);

	rt_fprintf(stderr, "T1 waits for T2 & T3\n");
	rt_task_join(& task_2);
	rt_task_join(& task_3);

	rt_fprintf(stderr, "T1 quits\n");
}


void thread_2(void * unused)
{
	rt_fprintf(stderr, "    T2 starts\n");
	rt_fprintf(stderr, "    T2 works\n");
	rt_timer_spin(1500000000LLU);
	rt_fprintf(stderr, "    T2 quits\n");
}


void thread_3(void * unused)
{
	rt_fprintf(stderr, "        T3 starts\n");

	rt_fprintf(stderr, "        T3 waits for the mutex\n");
	rt_mutex_acquire(& mutex, TM_INFINITE);
	rt_fprintf(stderr, "        T3 holds the mutex\n");

	rt_fprintf(stderr, "        T3 works\n");
	rt_timer_spin(1500000000LLU);

	rt_fprintf(stderr, "        T3 releases the mutex\n");
	rt_mutex_release(& mutex);
	rt_fprintf(stderr, "        T3 quits\n");
}



int main(int argc, char * argv [])
{
	int err;
	RT_TASK task;

	mlockall(MCL_CURRENT|MCL_FUTURE);

	if ((err = rt_mutex_create(& mutex, "Mutex-02")) != 0) {
		fprintf(stderr, "rt_mutex_create:%s\n",
		         strerror(-err));
		exit(EXIT_FAILURE);
	}

	if ((err = rt_task_spawn(& task, NULL, 0, 10,
	                  T_JOINABLE, thread_1, NULL) != 0)) {
		fprintf(stderr, "rt_task_spawn:%s\n",
		        strerror(-err));
		exit(EXIT_FAILURE);
	}

	rt_task_join(& task);
	rt_mutex_delete(& mutex);
	return 0;
}
