// ---------------------------------------------------------------------
// exemple-semaphore-01.c
// Fichier d'exemple du livre "Solutions Temps-Reel sous Linux"
// (C) 2012-2015 - Christophe BLAESS
// http://christophe.blaess.fr
// ---------------------------------------------------------------------

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/mman.h>

#include <alchemy/sem.h>
#include <alchemy/task.h>
#include <alchemy/timer.h>

#define NB_TASKS 6
static RT_SEM  sem;
static int with_semaphore[NB_TASKS];

void thread_function (void * arg)
{
	int i;
	int num = (int) arg;

	for (i = 0; i < 4; i ++) {
		with_semaphore[num] = 0;
		rt_sem_p(& sem, TM_INFINITE);
		with_semaphore[num] = 1;
		sleep(2);
		rt_sem_v(& sem);
	}
	with_semaphore[num] = -1;
}


void observer(void * unused)
{
	int i;
	rt_printf("With semaphore    Without semaphore\n");
	while (1) { 
		for (i = 0; i < NB_TASKS; i ++)
			if (with_semaphore[i] == 1)
				rt_printf("%d ", i);
		rt_printf ("          ");
		for (i = 0; i < NB_TASKS; i ++)
			if (with_semaphore[i] == 0)
				rt_printf("%d ", i);
		rt_printf("\n");
		rt_task_sleep(1000000000);
	}
}


int main(int argc, char * argv[])
{
	int i;
	int err;
	RT_TASK task[NB_TASKS];
	RT_TASK observ;

	mlockall(MCL_CURRENT|MCL_FUTURE);

	err = rt_sem_create(& sem, "Sem-01",
	                    NB_TASKS-2, S_PRIO);
	if (err != 0) {
		fprintf(stderr, "rt_sem_create:%s\n",
		         strerror(-err));
		exit(EXIT_FAILURE);
	}

	for (i = 0; i < NB_TASKS; i ++) {
		err = rt_task_spawn(& task[i], NULL, 
		                    0, 99, T_JOINABLE,
		                    thread_function, (void *) i);
		if (err != 0) {
			fprintf(stderr, "rt_task_spawn:%s\n",
			        strerror(-err));
			exit(EXIT_FAILURE);
		}
	}

	err = rt_task_spawn(& observ, NULL, 0, 99,
	             T_JOINABLE, observer, NULL);
	if (err != 0) {
		fprintf(stderr, "rt_task_spawn:%s\n",
		        strerror(-err));
		exit(EXIT_FAILURE);
	}

	for (i  = 0; i < NB_TASKS; i ++)
		rt_task_join(& task[i]);

	rt_task_delete(& observ);
	rt_sem_delete(& sem);

	return EXIT_SUCCESS;
}


