// ---------------------------------------------------------------------
// exemple-sigdebug-02.c
// Fichier d'exemple du livre "Solutions Temps-Reel sous Linux"
// (C) 2012-2015 - Christophe BLAESS
// http://christophe.blaess.fr
// ---------------------------------------------------------------------

#include <execinfo.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/mman.h>

#include <alchemy/task.h>


void hello_world_function (void * unused)
{
	int i;

	for (i = 5; i > 0; i --) {
		rt_fprintf(stderr, "Calling fprintf() in %d s.\n", i);
		rt_task_sleep(1000000000LL);
	}
	rt_fprintf(stderr, "Calling fprintf() now.\n");
	
	fprintf(stdout, "Hello World!\n");

	for (;;) {
		rt_fprintf(stderr, "fprintf() was called.\n");
		rt_task_sleep(1000000000LL);
	}
}


void sigdebug_handler(int signum, siginfo_t *info, void *unused)
{
	switch(sigdebug_reason(info)) {
		case SIGDEBUG_NOMLOCK:
			rt_printf("SIGDEBUG received: mlockall() missing.\n");
			break;
		default:
		case SIGDEBUG_UNDEFINED:
			rt_printf("SIGDEBUG received: dndefined reason.\n");
			break;
		case SIGDEBUG_WATCHDOG:
			rt_printf("SIGDEBUG received: watchdog alarm.\n");
			break;
			
		case SIGDEBUG_MIGRATE_SIGNAL:
			rt_printf("SIGDEBUG received: signal received.\n");
			break;

		case SIGDEBUG_MIGRATE_SYSCALL:
			rt_printf("SIGDEBUG received: system call.\n");
			break;

		case SIGDEBUG_MIGRATE_FAULT:
			rt_printf("SIGDEBUG received: system fault.\n");
			break;

		case SIGDEBUG_MIGRATE_PRIOINV:
			rt_printf("SIGDEBUG received: priority inversion.\n");
			break;
	}
}


int main(void)
{
	int err;
	RT_TASK task;
	struct sigaction action;

	mlockall(MCL_CURRENT|MCL_FUTURE);

	action.sa_sigaction = sigdebug_handler;
	sigfillset(&(action.sa_mask));
	action.sa_flags = SA_SIGINFO;
	sigaction(SIGDEBUG, &action, NULL);

	if ((err = rt_task_spawn(& task, "Sigdebug 02",
	                         0, 55, T_JOINABLE | T_WARNSW,
	                         hello_world_function, NULL)) != 0) {
		fprintf(stderr, "rt_task_spawn: %s\n", strerror(-err));
		exit(EXIT_FAILURE);
	}
	rt_task_join(& task);
	return 0;
}

