// ---------------------------------------------------------------------
// exemple-read-write.c
// Fichier d'exemple du livre "Solutions Temps-Reel sous Linux"
// (C) 2012-2015 - Christophe BLAESS
// http://christophe.blaess.fr
// ---------------------------------------------------------------------

#include <linux/device.h>
#include <linux/miscdevice.h>
#include <linux/module.h>
#include <linux/mutex.h>
#include <linux/fs.h>
#include <linux/uaccess.h>
#include <asm/uaccess.h>



#define MY_DATA_MAX_LENGTH 4096
static char my_data [MY_DATA_MAX_LENGTH];
static int  my_data_length = 0;
DEFINE_MUTEX(my_data_mutex);



static ssize_t my_driver_read(struct file * filp, char * __user u_buffer,
                        size_t lg, loff_t * offset)
{
	if (mutex_lock_interruptible(&my_data_mutex) != 0)
		return -ERESTARTSYS;

	if (lg > my_data_length)
		lg = my_data_length;

	if (lg > 0) {
		if (copy_to_user(u_buffer, my_data, lg) != 0) {
			mutex_unlock(&my_data_mutex);
			return -EFAULT;
		}
		my_data_length -= lg;
		if (my_data_length > 0)
			memmove(my_data, & my_data[lg], my_data_length);
	}
	mutex_unlock(&my_data_mutex);
	return lg;
}


static ssize_t my_driver_write(struct file * filp, const char * u_buffer,
                        size_t lg, loff_t * offset)
{
	if (mutex_lock_interruptible(&my_data_mutex) != 0)
		return -ERESTARTSYS;

	if (lg > (MY_DATA_MAX_LENGTH - my_data_length))
		lg = MY_DATA_MAX_LENGTH - my_data_length;

	if (lg > 0) {
		if (copy_from_user(&my_data[my_data_length],
		                   u_buffer, lg) != 0) {
			mutex_unlock(&my_data_mutex);
			return -EFAULT;
		}
		my_data_length += lg;
	}
	mutex_unlock(&my_data_mutex);
	return lg;
}



static struct file_operations my_driver_fops = {
	.owner   =  THIS_MODULE,
	.read    =  my_driver_read,
	.write   =  my_driver_write,
};



static struct miscdevice my_driver_misc = {
	.minor   = MISC_DYNAMIC_MINOR,
	.name    = THIS_MODULE->name,
	.fops    = &my_driver_fops,
};



static int __init my_driver_02_init (void)
{
	return misc_register(&my_driver_misc);
}



static void __exit my_driver_02_exit (void)
{
	misc_deregister(&my_driver_misc);
}



module_init(my_driver_02_init);
module_exit(my_driver_02_exit);
MODULE_LICENSE("GPL");
