// ---------------------------------------------------------------------
// exemple-read-irq.c
// Fichier d'exemple du livre "Solutions Temps-Reel sous Linux"
// (C) 2012-2015 - Christophe BLAESS
// http://christophe.blaess.fr
// ---------------------------------------------------------------------

#include <linux/interrupt.h>
#include <linux/device.h>
#include <linux/miscdevice.h>
#include <linux/module.h>
#include <linux/mutex.h>
#include <linux/fs.h>
#include <linux/uaccess.h>
#include <asm/uaccess.h>


#define MY_DATA_MAX  4096
static long int my_data[MY_DATA_MAX];
static int  my_data_count = 0;
static spinlock_t  my_data_spinlock;
static DECLARE_WAIT_QUEUE_HEAD(my_data_wq);


#define KBUFFER_SIZE  64

static ssize_t my_driver_read(struct file * filp, char * __user u_buffer,
                        size_t length, loff_t * offset)
{
	char k_buffer[KBUFFER_SIZE];
	unsigned long irqs;

	spin_lock_irqsave(&my_data_spinlock, irqs);

	while (my_data_count == 0) {
		spin_unlock_irqrestore(&my_data_spinlock, irqs);
		if (wait_event_interruptible(my_data_wq,
		                    (my_data_count != 0)) != 0) {
			return -ERESTARTSYS;
		}
		spin_lock_irqsave(&my_data_spinlock, irqs);
	}

	snprintf(k_buffer, KBUFFER_SIZE - 1, "%ld\n", my_data[0]);
	if (length < (strlen(k_buffer)+1)) {
		spin_unlock_irqrestore(&my_data_spinlock, irqs);
		return -ENOMEM;
	}

	my_data_count --;
	if (my_data_count > 0)
		memmove(my_data, & (my_data[1]), my_data_count * sizeof(long int));

	spin_unlock_irqrestore(&my_data_spinlock, irqs);

	if (copy_to_user(u_buffer, k_buffer, strlen(k_buffer)+1) != 0)
		return -EFAULT;
	return strlen(k_buffer);
}



static struct file_operations my_driver_fops = {
	.owner   =  THIS_MODULE,
	.read    =  my_driver_read,
};



static int irq_number = 1;
module_param(irq_number, int, 0444);

static irqreturn_t my_irq_handler(int irq, void * ident)
{
	spin_lock(&my_data_spinlock);
	if (my_data_count < MY_DATA_MAX) {
		my_data[my_data_count] = jiffies;
		my_data_count ++;
	}
	spin_unlock(&my_data_spinlock);
	wake_up_interruptible(&my_data_wq);
	return IRQ_HANDLED;
}



static struct miscdevice my_driver_misc = {
	.minor   = MISC_DYNAMIC_MINOR,
	.name    = THIS_MODULE->name,
	.fops    = &my_driver_fops,
};



static int __init my_driver_03_init (void)
{
	int err;

	spin_lock_init(&my_data_spinlock);

	err = request_irq(irq_number, my_irq_handler,
	                  IRQF_SHARED, THIS_MODULE->name, THIS_MODULE->name);
	if (err != 0)
		return err;

	err = misc_register(&my_driver_misc);

	if (err != 0) {
		free_irq(irq_number, THIS_MODULE->name);
	}

	return err; 
}



static void __exit my_driver_03_exit (void)
{
	free_irq(irq_number, THIS_MODULE->name);
	misc_deregister(&my_driver_misc);
}



module_init(my_driver_03_init);
module_exit(my_driver_03_exit);
MODULE_LICENSE("GPL");
