// ---------------------------------------------------------------------
// exemple-read-irq.c
// Fichier d'exemple du livre "Solutions Temps-Reel sous Linux"
// (C) 2012-2015 - Christophe BLAESS
// http://christophe.blaess.fr
// ---------------------------------------------------------------------

#include <linux/interrupt.h>
#include <linux/module.h>
#include <linux/gpio.h>



static int gpio_in = 23;
module_param(gpio_in, int, 0644);

static int gpio_out = 24;
module_param(gpio_out, int, 0644);


static irqreturn_t my_irq_handler(int irq, void * ident)
{
	static int value = 0;

	value = 1 - value;
	gpio_set_value(gpio_out, value);
	return IRQ_HANDLED;
}



static int __init my_driver_05_init (void)
{
	int err;

	err = gpio_request(gpio_in, THIS_MODULE->name);
	if (err != 0)
		goto out_return;

	err = gpio_request(gpio_out, THIS_MODULE->name);
	if (err != 0)
		goto out_free_gpio_in;

	err = gpio_direction_input(gpio_in);
	if (err != 0)
		goto out_free_gpio_out;

	err = gpio_direction_output(gpio_out, 0);
	if (err != 0)
		goto out_free_gpio_out;

	err = request_irq(gpio_to_irq(gpio_in), my_irq_handler,
	                  IRQF_SHARED | IRQF_TRIGGER_RISING,
	                  THIS_MODULE->name, THIS_MODULE->name);
	if (err != 0)
		goto out_free_gpio_out;

	return 0;

	out_free_gpio_out:
		gpio_free(gpio_out);
	out_free_gpio_in:
		gpio_free(gpio_in);
	out_return:
		return err;
}



static void __exit my_driver_05_exit (void)
{
	free_irq(gpio_to_irq(gpio_in), THIS_MODULE->name);
	gpio_free(gpio_out);
	gpio_free(gpio_in);
}



module_init(my_driver_05_init);
module_exit(my_driver_05_exit);
MODULE_LICENSE("GPL");
