// ---------------------------------------------------------------------
// exemple-rtdm-read-write.c
// Fichier d'exemple du livre "Solutions Temps-Reel sous Linux"
// (C) 2012 - Christophe BLAESS
// http://christophe.blaess.fr
// ---------------------------------------------------------------------

#include <linux/version.h>
#include <linux/device.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/cdev.h>
#include <linux/fs.h>
#include <asm/uaccess.h>

#include <rtdm/rtdm.h>
#include <rtdm/driver.h>


#define MY_DATA_SIZE  4096
static char my_data [MY_DATA_SIZE];
static int  my_data_end = 0;
static rtdm_mutex_t my_data_mtx;



static int my_open_function(struct rtdm_fd *fd, int flags)
{
	rtdm_printk(KERN_INFO "%s.%s()\n", THIS_MODULE->name, __FUNCTION__);
	return 0;
}



static void my_close_function(struct rtdm_fd *fd)
{
	rtdm_printk(KERN_INFO "%s.%s()\n", THIS_MODULE->name, __FUNCTION__);
}



static int my_read_nrt_function  (struct rtdm_fd *fd, void __user *buffer, size_t lg)
{
	rtdm_printk(KERN_INFO "%s.%s()\n", THIS_MODULE->name, __FUNCTION__);

	rtdm_mutex_lock(&my_data_mtx);

	if (lg > my_data_end)
		lg = my_data_end;

	if (lg > 0) {
		if (rtdm_safe_copy_to_user(fd, buffer, my_data, lg) != 0) {
			rtdm_mutex_unlock(&my_data_mtx);
			return -EFAULT;
		}
		my_data_end -= lg;
		if (my_data_end > 0)
			memmove(my_data, &my_data[lg], my_data_end);
	}
	rtdm_mutex_unlock(&my_data_mtx);
	return lg;
}



static int my_write_nrt_function(struct rtdm_fd *fd, const void __user *buffer, size_t lg)
{
	rtdm_mutex_lock(&my_data_mtx);

	if (lg > (MY_DATA_SIZE - my_data_end))
		lg = MY_DATA_SIZE - my_data_end;

	if (lg > 0) {
		if (rtdm_safe_copy_from_user(fd,
		                  &my_data[my_data_end],
		                  buffer, lg) != 0) {
			rtdm_mutex_unlock(&my_data_mtx);
			return -EFAULT;
		}
		my_data_end += lg;
	}
	rtdm_mutex_unlock(&my_data_mtx);
	return lg;
}



static struct rtdm_driver my_rt_driver = {

	.profile_info = RTDM_PROFILE_INFO(my_example, RTDM_CLASS_TESTING, 1, 1),

	.device_flags   = RTDM_NAMED_DEVICE,
	.device_count   = 1,
	.context_size   = 0,

	.ops = {
		.open      = my_open_function,
		.close     = my_close_function,
		.read_nrt  = my_read_nrt_function,
		.write_nrt = my_write_nrt_function,
	},
};


static struct rtdm_device my_rt_device = {

	.driver = &my_rt_driver,
	.label  = "rtdm_driver_%d",
};


static int __init my_module_init (void)
{
	rtdm_mutex_init(&my_data_mtx);
	rtdm_dev_register(&my_rt_device);
	return 0;
}



static void __exit my_module_exit (void)
{
	rtdm_dev_unregister(&my_rt_device);
	rtdm_mutex_destroy(&my_data_mtx);
}



module_init(my_module_init);
module_exit(my_module_exit);
MODULE_LICENSE("GPL");
