// ---------------------------------------------------------------------
// exemple-rtdm-irq.c
// Fichier d'exemple du livre "Solutions Temps-Reel sous Linux"
// (C) 2012-2015 - Christophe BLAESS
// http://christophe.blaess.fr
// ---------------------------------------------------------------------

#include <linux/interrupt.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/gpio.h>

#include <asm/uaccess.h>

#include <rtdm/rtdm.h>
#include <rtdm/driver.h>


static int gpio_in = 23;
module_param(gpio_in, int, 0644);

static int gpio_out = 24;
module_param(gpio_out, int, 0644);


static int my_irq_handler(rtdm_irq_t * irq)
{
	static int value = 0;

	gpio_set_value(gpio_out, value);
	value = 1 - value;
	return RTDM_IRQ_HANDLED;
}


static rtdm_irq_t gpio_in_irq;


static int __init my_module_init (void)
{
	int err;
	int irq_number = gpio_to_irq(gpio_in);

	if ((err = gpio_request(gpio_in, THIS_MODULE->name)) != 0)
		return err;

	if ((err = gpio_direction_input(gpio_in)) != 0)
		goto gpio_free_gpio_in;

	if ((err = gpio_request(gpio_out, THIS_MODULE->name)) != 0)
		goto gpio_free_gpio_in;

	if ((err = gpio_direction_output(gpio_out, 1)) != 0)
		goto gpio_free_gpio_out;

	err = rtdm_irq_request(&gpio_in_irq, 
	                       irq_number,
	                       gpio_in_irq_handler, 
	                       RTDM_IRQTYPE_EDGE | IRQF_TRIGGER_RISING,
	                       "GPIO IN", NULL);
	if (err != 0)
		goto gpio_free_gpio_out;

	rtdm_irq_enable(&gpio_in_irq);

	return 0;

	out_free_gpio_out:
	gpio_free(gpio_out);

	out_free_gpio_in:

	gpio_free(gpio_in);
	return err;
}



static void __exit my_module_exit (void)
{
	rtdm_irq_disable(&gpio_in_irq);
	rtdm_irq_free(&gpio_in_irq);
	gpio_free(gpio_out);
	gpio_free(gpio_in);
}


module_init(my_module_init);
module_exit(my_module_exit);
MODULE_LICENSE("GPL");
